package org.apache.bcel.classfile;

import java.util.stream.Stream;
import java.util.Iterator;
import java.io.DataOutputStream;
import java.util.Arrays;
import java.io.IOException;
import java.io.DataInput;

public class MethodParameters extends Attribute implements Iterable<MethodParameter>
{
    private static final MethodParameter[] EMPTY_METHOD_PARAMETER_ARRAY;
    private MethodParameter[] parameters;
    
    MethodParameters(final int nameIndex, final int length, final DataInput input, final ConstantPool constantPool) throws IOException {
        super((byte)21, nameIndex, length, constantPool);
        this.parameters = MethodParameters.EMPTY_METHOD_PARAMETER_ARRAY;
        final int parameterCount = input.readUnsignedByte();
        this.parameters = new MethodParameter[parameterCount];
        for (int i = 0; i < parameterCount; ++i) {
            this.parameters[i] = new MethodParameter(input);
        }
    }
    
    @Override
    public void accept(final Visitor v) {
        v.visitMethodParameters(this);
    }
    
    @Override
    public Attribute copy(final ConstantPool constantPool) {
        final MethodParameters c = (MethodParameters)this.clone();
        Arrays.setAll(c.parameters = new MethodParameter[this.parameters.length], i -> this.parameters[i].copy());
        c.setConstantPool(constantPool);
        return c;
    }
    
    @Override
    public void dump(final DataOutputStream file) throws IOException {
        super.dump(file);
        file.writeByte(this.parameters.length);
        for (final MethodParameter parameter : this.parameters) {
            parameter.dump(file);
        }
    }
    
    public MethodParameter[] getParameters() {
        return this.parameters;
    }
    
    @Override
    public Iterator<MethodParameter> iterator() {
        return Stream.of(this.parameters).iterator();
    }
    
    public void setParameters(final MethodParameter[] parameters) {
        this.parameters = parameters;
    }
    
    static {
        EMPTY_METHOD_PARAMETER_ARRAY = new MethodParameter[0];
    }
}
